#ifndef _YYR_FFT_H_  
#define _YYR_FFT_H_  
  
#include <math.h>  
  
using namespace std;  
  
namespace YYR_FFT  
{  
    //帴ֻͣdoublelong doubleܱΪ64λ  
    typedef double ComplexType; //Ĭʹdouble  
    typedef complex<ComplexType> Complex;   
    //г  
    const ComplexType PI = 3.14159265358979323846;  
    /*----------------ṩڲõӺ----------------*/  
  
    //ĿԪصĸǷ2^M|M >= 2  
    template<typename T>  
    inline bool CheckRange(const T Begin, const T End)  
    {    
        if(!(Begin < End))  
        {  
            return false;  
        }  
  
        size_t N = End - Begin;  
  
        if (N < 2)  
        {  
            return false;  
        }  
  
        while (N > 1)  
        {  
            if (N % 2 > 0)  
            {  
                return false;  
            };  
            N /= 2;  
        }  
  
        return true;  
    }  
    //  
    template<typename T>  
    inline void CompareAndSwap(T Left, T Right)  
    {  
        if (Left != Right)  
        {  
            swap(*Left, *Right);  
        }  
    }  
    //  
    //Wֵ  
    inline Complex CalculateW(const size_t K, const size_t N)  
    {  
        ComplexType Alpha = -2 * PI * K / N;  
        return Complex(cos(Alpha), sin(Alpha));  
    };  
    //Wֵ,任  
    inline Complex CalculateRW(const size_t K, const size_t N)  
    {  
        ComplexType Alpha = 2 * PI * K / N;  
        return Complex(cos(Alpha), sin(Alpha));  
    };  
    //  
    template<typename T>  
    void _FFT(T Begin, T End)  
    {  
        typename T::value_type X1;  
        typename T::value_type X2;  
          
        size_t N = End - Begin;  
  
        T Left  = Begin;  
        T Right;  
        for (size_t I = 0; I < N / 2; ++I, Left += 2)  
        {  
            Right = Left + 1;  
  
            X1 = *Left;  
            X2 = *Right;  
            *Left  = X1 + X2;  
            *Right = X1 - X2;  
        }  
  
        for (size_t GroupLength = 4; GroupLength <= N; GroupLength *= 2)  
        {  
            size_t GroupTotal = N / GroupLength;  
            size_t HalfGroupLen = GroupLength / 2;  
  
            for (size_t I = 0; I < GroupTotal; ++I)  
            {  
                Left  = Begin + I * GroupLength;  
                for (size_t J = 0; J < HalfGroupLen; ++J, ++Left)  
                {  
                    Right = Left + HalfGroupLen;  
                      
                    X1 = *Left;  
                    X2 = (*Right) * CalculateW(J, GroupLength);  
  
                    *Left  = X1 + X2;  
                    *Right = X1 - X2;  
                }  
            }  
        }  
    }  
    //  
    template<typename T>  
    void _RaderSort(T Begin, T End)  
    {  
        size_t TotalLength = (End - Begin);  
        size_t N = TotalLength / 2;  
        size_t I = 1;  
        size_t J = N;  
        size_t NN;  
  
        --TotalLength;  
        for (; I < TotalLength; ++I)  
        {  
            if (I < J)  
            {  
                swap(*(Begin+I), *(Begin+J));  
            }  
  
            NN = N;  
            while (true)  
            {  
                if (NN > J)  
                {  
                    J += NN;  
                    break;  
                }  
                else  
                {  
                    J  -= NN;  
                    NN /= 2;  
                }  
            }  
        }  
    }  
  
    /*---------------ṩⲿõĺ----------------*/  
  
    // (ҿ)ĸɢҶ任(ʱ任DIT-FFT)ʹÿٸҶ任㷨Ӷ  
    // O(N) = N * LogN.    
    // Լ: ĿĸĸҪ2^M(M >= 1).  任ʧܷfalse.  
    // 任ɹ󷵻true,Ŀ䱣  

 //׵㷨,Ŀ(ҿ)Ԫؽеλ,ӶλO(N) = N;  
    template<typename T>  
    bool RaderSort(T Begin, T End)  
    {  
        if (CheckRange(Begin, End) == false)  
        {  
            return false;  
        }  
  
        _RaderSort(Begin, End);  
        return true;  
    }  


    template<typename T>  
    bool FFT(T Begin, T End)  
    {  
        if (CheckRange(Begin, End) == false)  
        {  
            return false;  
        }  
  
        RaderSort(Begin, End);  
  
        _FFT(Begin, End);  
          
        return true;  
    }  
    //  
    // (ҿ)ĸɢҶ任(IFFT)ʹÿٸҶ任㷨Ӷ  
    // O(N) = N * LogN.    
    // Լ: ĿĸĸҪ2^M(M >= 1).  任ʧܷfalse.  
    // 任ɹ󷵻true,Ŀ䱣  
    template<typename T>  
    bool IFFT(T Begin, T End)  
    {  
        if (CheckRange(Begin, End) == false)  
        {  
            return false;  
        }  
  
        TranToConjugate(Begin, End);  
  
        RaderSort(Begin, End);  
  
        _FFT(Begin, End);  
  
        TranToConjugate(Begin, End);  
          
        size_t N = End - Begin;  
        for (T Iter = Begin; Iter < End; ++Iter)  
        {  
            (*Iter) /= N;  
        }  
  
        return true;  
    }  
  
   
  
    //Ŀ(ҿ)ڵĸ޸Ϊ  
    template<typename T>  
    void TranToConjugate(T Begin, T End)  
    {  
        for (T Iter = Begin; Iter < End; ++Iter)  
        {  
            Iter->_Val[1] = 0 - (Iter->_Val[1]);  
        }  
    }  
    //ȫ㷨ĸҶ任ӶΪ   
    //O(N) = N * N  
    template<typename T, typename T2>  
    void F(T Begin, T End, T2& V)  
    {  
        size_t  Total = End - Begin;  
        Complex Temp;  
        for (size_t K = 0; K < Total; ++K)  
        {  
            Temp = Complex(0, 0);  
            for (size_t N = 0; N < Total; ++N)  
            {  
                Temp += ((*(Begin + N)) * (CalculateW(K*N, Total)));  
            }  
            V.push_back(Temp);  
        }  
    }  
    //ȫ㷨ĸҶ任ӶΪ   
    //O(N) = N * N  
    template<typename T, typename T2>  
    void IF(T Begin, T End, T2& V)  
    {  
        size_t  Total = End - Begin;  
        Complex Temp;  
        for (size_t K = 0; K < Total; ++K)  
        {  
            Temp = Complex(0, 0);  
            for (size_t N = 0; N < Total; ++N)  
            {  
                Temp += ((*(Begin + N)) * (CalculateRW(K*N, Total)));  
            }  
            Temp /= Total;  
            V.push_back(Temp);  
        }  
    }  
}  
  
#endif // !_YYR_FFT_H_  
